import csv
import datetime
import requests
from lxml import etree as ET

# Function to create CDATA element
def create_cdata_element(tag, text):
    element = ET.Element(tag)
    element.text = ET.CDATA(text)
    return element

# URL of the CSV file
csv_url = 'https://dkkapusta1997.usermd.net/Filip/your_input.csv'  # Replace with the actual URL

# Define the XML output file name
xml_file = 'skroutz.xml'

# Define the formula parameters
base_price = 2  # Adjust this value as needed
price_factor = 1.7  # Adjust this value as needed

# Specify the threshold quantity below which the quantity should be set to 0
threshold_quantity = 10  # Adjust this value as needed

# Download the CSV file from the URL
response = requests.get(csv_url)

# Check if the request was successful
if response.status_code == 200:
    # Save the downloaded CSV content to a local file
    with open('downloaded_input.csv', 'wb') as csv_file:
        csv_file.write(response.content)
    
    # Open the downloaded CSV file for processing
    with open('downloaded_input.csv', 'r', encoding='utf-8') as csvfile:
        csvreader = csv.DictReader(csvfile)
        
        # Create the root element of the XML
        shumee = ET.Element('shumee')
        
        # Add the created_at element with the current date and time
        created_at = ET.SubElement(shumee, 'created_at')
        created_at.text = datetime.datetime.now().strftime('%Y-%m-%d %H:%M')
        
        # Create the products element
        products = ET.SubElement(shumee, 'products')
        
        for row in csvreader:
            product = ET.SubElement(products, 'product')
            
            # Define a mapping between CSV column headers and XML elements
            mapping = {
                'SKU': 'id',
                'Title': 'name',
                'Image 1': 'link',
                'Image 2': 'additional_imageurl',
                'Image 3': 'additional_imageurl',
                'Image 4': 'additional_imageurl',
                'Image 5': 'additional_imageurl',
                'Image 6': 'additional_imageurl',
                'Category': 'category',
                'Brand': 'manufacturer',
                'EAN': 'ean',
                'Weight': 'weight',
                'Description': 'description',
                'Stock': 'quantity'
            }
            
            for csv_header, xml_element in mapping.items():
                value = row.get(csv_header, '')
                
                # Create XML element and set its text
                if xml_element in ['name', 'link', 'additional_imageurl', 'description', 'category', 'manufacturer']:
                    element = create_cdata_element(xml_element, value)
                else:
                    element = ET.Element(xml_element)
                    element.text = value
                
                if xml_element == 'link':
                    # Add missing <image> element with CDATA just after <link>
                    product.append(element)
                    product.append(create_cdata_element('image', value))
                else:
                    product.append(element)
            
            # Calculate the price using the specified formula
            calculated_price = (float(base_price) + float(row.get('B2B price', '0'))) * price_factor
            
            # Check if the quantity is lower or equal to the threshold and set it to 0 if so
            if calculated_price <= threshold_quantity:
                quantity = ET.SubElement(product, 'quantity')
                quantity.text = '0'
            else:
                # Add VAT element with a fixed value
                vat = ET.SubElement(product, 'vat')
                vat.text = '24.00'
                
                # Add calculated price as the B2B price
                price_with_vat = ET.SubElement(product, 'price_with_vat')
                price_with_vat.text = '{:.2f}'.format(calculated_price)
                
                # Add availability element with a fixed value
                availability = ET.SubElement(product, 'availability')
                availability.text = 'Delivery 1 to 3 days'
                
                # Add <mpn> element with CDATA
                mpn = create_cdata_element('mpn', row.get('SKU', ''))
                product.append(mpn)
        
        # Create the XML tree and write it to the output file
        tree = ET.ElementTree(shumee)
        tree.write(xml_file, encoding='utf-8', xml_declaration=True)
        
        print(f'XML file "{xml_file}" has been created successfully.')
else:
    print(f'Failed to download the CSV file from the URL: {csv_url}')
